#ifdef GL_ES
precision mediump float;
#endif

#define circle_size .5
#define orbit_speed 1.0
#define orbit_radius .43
#define gravity_constant 20.
#define speed_of_light .2
#define color_scale .001
#define color_offset .06

// I wanted to simulate gravity waves from two orbiting black holes,
// but then I didn't manage to wrap my head around the calculation of
// the gravity from an orbiting black hole, so I went with the simplified
// version where you check the distance to the black hole, and 
// calculate the gravity based on the black hole n seconds in the past
// where n is proportional to the distance.

// it is very wrong but it look cool ¯\_(ツ)_/¯

uniform float u_time;
uniform vec2 u_resolution;

float circleSDF(vec2 center, float radius, vec2 position)
{
  return length(position - center) - radius;
}

vec2 getGravityVector(vec2 gravityPos, vec2 pos) {
  // get the size of the gravitational field, when at position `pos` next to body of mass 1 at position `gravityPos`
  float gravity = gravity_constant / pow((length(pos - gravityPos)), 2.);
  return (gravityPos - pos) * gravity;
}

float twoCirclesSDF(vec2 pos, float time) 
{
  vec2 hole1now = vec2(-sin(orbit_speed * time) * orbit_radius, -cos(orbit_speed * time) * orbit_radius);
  vec2 hole2now = vec2(sin(orbit_speed * time) * orbit_radius, cos(orbit_speed * time) * orbit_radius);
  float hole1delay = length(hole1now - pos) / speed_of_light;
  float hole2delay = length(hole2now - pos) / speed_of_light;
  vec2 hole1relative = vec2(-sin(orbit_speed * time - hole1delay) * orbit_radius, -cos(orbit_speed * time - hole2delay) * orbit_radius);
  vec2 hole2relative = vec2(sin(orbit_speed * time - hole2delay) * orbit_radius, cos(orbit_speed * time - hole2delay) * orbit_radius);


  vec2 grav1 = getGravityVector(hole1relative, pos);
  vec2 grav2 = getGravityVector(hole2relative, pos);
  vec2 grav = grav1 + grav2;
  return length(grav) * color_scale;

}

vec2 sphere(vec2 pos) {
  return pos * asin(length(pos) / circle_size) / length(pos);
}

void mainImage( out vec4 fragColor, in vec2 fragCoord )
{
  // Normalized pixel coordinates (from 0 to 1)
  vec2 uv = (fragCoord - .5 * u_resolution.xy)/min(u_resolution.x, u_resolution.y);
  uv = sphere(uv);
  float r = twoCirclesSDF(uv, u_time);
  float g = twoCirclesSDF(uv, u_time + color_offset);
  float b = twoCirclesSDF(uv, u_time + color_offset * 2.);


  fragColor = vec4(r, g, b, 1.);
}

void main() {
    mainImage(gl_FragColor, gl_FragCoord.xy);
}
